@@ -1,3 +1,31 @@
+/*
+Copyright (c) 1982, 1986, 1993
+The Regents of the University of California.  All rights reserved.
+
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions
+are met:
+1. Redistributions of source code must retain the above copyright
+notice, this list of conditions and the following disclaimer.
+2. Redistributions in binary form must reproduce the above copyright
+notice, this list of conditions and the following disclaimer in the
+documentation and/or other materials provided with the distribution.
+3. Neither the name of the University nor the names of its contributors
+may be used to endorse or promote products derived from this software
+without specific prior written permission.
+
+THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+SUCH DAMAGE.
+ */
 #ifndef _SYS_UNISTD_H
 #define _SYS_UNISTD_H
 
@@ -21,9 +49,7 @@ int	access (const char *__path, int __amode);
 unsigned  alarm (unsigned __secs);
 int     chdir (const char *__path);
 int     chmod (const char *__path, mode_t __mode);
-#if !defined(__INSIDE_CYGWIN__)
 int     chown (const char *__path, uid_t __owner, gid_t __group);
-#endif
 #if __BSD_VISIBLE || (__XSI_VISIBLE >= 4 && __POSIX_VISIBLE < 200112)
 int     chroot (const char *__path);
 #endif
@@ -50,7 +76,7 @@ int     dup3 (int __fildes, int __fildes2, int flags);
 int	eaccess (const char *__path, int __mode);
 #endif
 #if __XSI_VISIBLE
-void	encrypt (char *__block, int __edflag);
+void	encrypt (char *, int);
 #endif
 #if __BSD_VISIBLE || (__XSI_VISIBLE && __XSI_VISIBLE < 500)
 void	endusershell (void);
@@ -79,11 +105,9 @@ int     fchdir (int __fildes);
 #if __POSIX_VISIBLE >= 199309
 int     fchmod (int __fildes, mode_t __mode);
 #endif
-#if !defined(__INSIDE_CYGWIN__)
 #if __BSD_VISIBLE || __XSI_VISIBLE >= 4 || __POSIX_VISIBLE >= 200809
 int     fchown (int __fildes, uid_t __owner, gid_t __group);
 #endif
-#endif
 #if __ATFILE_VISIBLE
 int	fchownat (int __dirfd, const char *__path, uid_t __owner, gid_t __group, int __flags);
 #endif
@@ -106,11 +130,9 @@ int	getdomainname  (char *__name, size_t __len);
 #if __BSD_VISIBLE
 int     getentropy (void *, size_t);
 #endif
-#if !defined(__INSIDE_CYGWIN__)
 gid_t   getegid (void);
 uid_t   geteuid (void);
 gid_t   getgid (void);
-#endif
 int     getgroups (int __gidsetsize, gid_t __grouplist[]);
 #if __BSD_VISIBLE || __XSI_VISIBLE >= 4
 long    gethostid (void);
@@ -135,9 +157,7 @@ pid_t   getppid (void);
 #if __POSIX_VISIBLE >= 200809 || __XSI_VISIBLE >= 4
 pid_t   getsid (pid_t);
 #endif
-#if !defined(__INSIDE_CYGWIN__)
 uid_t   getuid (void);
-#endif
 #if __BSD_VISIBLE || (__XSI_VISIBLE && __XSI_VISIBLE < 500)
 char *	getusershell (void);
 #endif
@@ -151,11 +171,9 @@ int     isatty (int __fildes);
 #if __BSD_VISIBLE
 int        issetugid (void);
 #endif
-#if !defined(__INSIDE_CYGWIN__)
 #if __BSD_VISIBLE || __XSI_VISIBLE >= 4 || __POSIX_VISIBLE >= 200809
 int     lchown (const char *__path, uid_t __owner, gid_t __group);
 #endif
-#endif
 int     link (const char *__path1, const char *__path2);
 #if __ATFILE_VISIBLE
 int     linkat (int __dirfd1, const char *__path1, int __dirfd2, const char *__path2, int __flags);
@@ -198,13 +216,11 @@ int	ruserok (const char *rhost, int superuser, const char *ruser, const char *lu
 #if __BSD_VISIBLE || (__XSI_VISIBLE >= 4 && __POSIX_VISIBLE < 200112)
 void *  sbrk (ptrdiff_t __incr);
 #endif
-#if !defined(__INSIDE_CYGWIN__)
 #if __BSD_VISIBLE || __POSIX_VISIBLE >= 200112
 int     setegid (gid_t __gid);
 int     seteuid (uid_t __uid);
 #endif
 int     setgid (gid_t __gid);
-#endif
 #if __BSD_VISIBLE
 int	setgroups (int ngroups, const gid_t *grouplist);
 #endif
@@ -215,14 +231,12 @@ int     setpgid (pid_t __pid, pid_t __pgid);
 #if __SVID_VISIBLE || __XSI_VISIBLE >= 500
 int     setpgrp (void);
 #endif
-#if (__BSD_VISIBLE || __XSI_VISIBLE >= 4) && !defined(__INSIDE_CYGWIN__)
+#if __BSD_VISIBLE || __XSI_VISIBLE >= 4
 int	setregid (gid_t __rgid, gid_t __egid);
 int	setreuid (uid_t __ruid, uid_t __euid);
 #endif
 pid_t   setsid (void);
-#if !defined(__INSIDE_CYGWIN__)
 int     setuid (uid_t __uid);
-#endif
 #if __BSD_VISIBLE || (__XSI_VISIBLE && __XSI_VISIBLE < 500)
 void	setusershell (void);
 #endif
@@ -259,7 +273,7 @@ extern int optreset;			/* getopt(3) external variable */
 pid_t   vfork (void);
 #endif
 
-#ifdef _COMPILING_NEWLIB
+#ifdef _LIBC
 /* Provide prototypes for most of the _<systemcall> names that are
    provided in newlib for some compilers.  */
 int     close (int __fildes);
@@ -533,7 +547,8 @@ int	unlinkat (int, const char *, int);
  *  confstr values per IEEE Std 1003.1, 2004 Edition
  */
 
-#ifdef __CYGWIN__	/* Only defined on Cygwin for now. */
+/* Only defined on Cygwin and RTEMS for now. */
+#if defined (__CYGWIN__) || defined(__rtems__)
 #define _CS_PATH                               0
 #define _CS_POSIX_V7_ILP32_OFF32_CFLAGS        1
 #define _CS_POSIX_V6_ILP32_OFF32_CFLAGS       _CS_POSIX_V7_ILP32_OFF32_CFLAGS
