@@ -1,5 +1,4 @@
 //go:build k210
-// +build k210
 
 package machine
 
@@ -23,11 +22,17 @@ type PinChange uint8
 // Pin modes.
 const (
 	PinInput PinMode = iota
-	PinInputPullUp
-	PinInputPullDown
+	PinInputPullup
+	PinInputPulldown
 	PinOutput
 )
 
+// Deprecated: use PinInputPullup and PinInputPulldown instead.
+const (
+	PinInputPullUp   = PinInputPullup
+	PinInputPullDown = PinInputPulldown
+)
+
 // FPIOA internal pull resistors.
 const (
 	fpioaPullNone fpioaPullMode = iota
@@ -89,10 +94,10 @@ func (p Pin) Configure(config PinConfig) {
 	case PinInput:
 		p.setFPIOAIOPull(fpioaPullNone)
 		input = true
-	case PinInputPullUp:
+	case PinInputPullup:
 		p.setFPIOAIOPull(fpioaPullUp)
 		input = true
-	case PinInputPullDown:
+	case PinInputPulldown:
 		p.setFPIOAIOPull(fpioaPullDown)
 		input = true
 	case PinOutput:
@@ -387,13 +392,16 @@ func (uart *UART) handleInterrupt(interrupt.Interrupt) {
 	uart.Receive(c)
 }
 
-func (uart *UART) WriteByte(c byte) {
+func (uart *UART) writeByte(c byte) error {
 	for uart.Bus.TXDATA.Get()&kendryte.UARTHS_TXDATA_FULL != 0 {
 	}
 
 	uart.Bus.TXDATA.Set(uint32(c))
+	return nil
 }
 
+func (uart *UART) flush() {}
+
 type SPI struct {
 	Bus *kendryte.SPI_Type
 }
@@ -517,7 +525,7 @@ type I2CConfig struct {
 func (i2c *I2C) Configure(config I2CConfig) error {
 
 	if config.Frequency == 0 {
-		config.Frequency = TWI_FREQ_100KHZ
+		config.Frequency = 100 * KHz
 	}
 
 	if config.SDA == 0 && config.SCL == 0 {
