@@ -1,7 +1,11 @@
-// +build stm32
+//go:build stm32
 
 package machine
 
+import "device/stm32"
+
+const deviceName = stm32.Device
+
 // Peripheral abstraction layer for the stm32.
 
 const (
@@ -15,6 +19,7 @@ const (
 	portH
 	portI
 	portJ
+	portK
 )
 
 // Peripheral operations sequence:
@@ -51,10 +56,27 @@ func (p Pin) Set(high bool) {
 	}
 }
 
-// Get returns the current value of a GPIO pin.
+// Get returns the current value of a GPIO pin when the pin is configured as an
+// input or as an output.
 func (p Pin) Get() bool {
 	port := p.getPort()
 	pin := uint8(p) % 16
 	val := port.IDR.Get() & (1 << pin)
 	return (val > 0)
 }
+
+// PortMaskSet returns the register and mask to enable a given GPIO pin. This
+// can be used to implement bit-banged drivers.
+func (p Pin) PortMaskSet() (*uint32, uint32) {
+	port := p.getPort()
+	pin := uint8(p) % 16
+	return &port.BSRR.Reg, 1 << pin
+}
+
+// PortMaskClear returns the register and mask to disable a given port. This can
+// be used to implement bit-banged drivers.
+func (p Pin) PortMaskClear() (*uint32, uint32) {
+	port := p.getPort()
+	pin := uint8(p) % 16
+	return &port.BSRR.Reg, 1 << (pin + 16)
+}
