@@ -555,8 +555,26 @@ func (v Value) Slice(i, j int) Value {
 		}
 
 	case Array:
-		// TODO(dgryski): can't do this yet because the resulting value needs type slice of v.elem(), not array of v.elem().
-		// need to be able to look up this "new" type so pointer equality of types still works
+		v.checkAddressable()
+		buf, length := buflen(v)
+		i, j := uintptr(i), uintptr(j)
+		if j < i || length < j {
+			slicePanic()
+		}
+
+		elemSize := v.typecode.underlying().elem().Size()
+
+		var hdr sliceHeader
+		hdr.len = j - i
+		hdr.cap = length - i
+		hdr.data = unsafe.Add(buf, i*elemSize)
+
+		sliceType := (*arrayType)(unsafe.Pointer(v.typecode.underlying())).slicePtr
+		return Value{
+			typecode: sliceType,
+			value:    unsafe.Pointer(&hdr),
+			flags:    v.flags,
+		}
 
 	case String:
 		i, j := uintptr(i), uintptr(j)
@@ -604,9 +622,26 @@ func (v Value) Slice3(i, j, k int) Value {
 		}
 
 	case Array:
-		// TODO(dgryski): can't do this yet because the resulting value needs type v.elem(), not array of v.elem().
-		// need to be able to look up this "new" type so pointer equality of types still works
+		v.checkAddressable()
+		buf, length := buflen(v)
+		i, j, k := uintptr(i), uintptr(j), uintptr(k)
+		if j < i || k < j || length < k {
+			slicePanic()
+		}
 
+		elemSize := v.typecode.underlying().elem().Size()
+
+		var hdr sliceHeader
+		hdr.len = j - i
+		hdr.cap = k - i
+		hdr.data = unsafe.Add(buf, i*elemSize)
+
+		sliceType := (*arrayType)(unsafe.Pointer(v.typecode.underlying())).slicePtr
+		return Value{
+			typecode: sliceType,
+			value:    unsafe.Pointer(&hdr),
+			flags:    v.flags,
+		}
 	}
 
 	panic("unimplemented: (reflect.Value).Slice3()")
@@ -913,11 +948,12 @@ func (v Value) MapKeys() []Value {
 	k := New(v.typecode.Key())
 	e := New(v.typecode.Elem())
 
-	keyType := v.typecode.Key().(*rawType)
-	isKeyStoredAsInterface := keyType.Kind() != String && !keyType.isBinary()
+	keyType := v.typecode.key()
+	keyTypeIsEmptyInterface := keyType.Kind() == Interface && keyType.NumMethod() == 0
+	shouldUnpackInterface := !keyTypeIsEmptyInterface && keyType.Kind() != String && !keyType.isBinary()
 
 	for hashmapNext(v.pointer(), it, k.value, e.value) {
-		if isKeyStoredAsInterface {
+		if shouldUnpackInterface {
 			intf := *(*interface{})(k.value)
 			v := ValueOf(intf)
 			keys = append(keys, v)
@@ -992,12 +1028,14 @@ func (v Value) MapRange() *MapIter {
 	}
 
 	keyType := v.typecode.key()
-	isKeyStoredAsInterface := keyType.Kind() != String && !keyType.isBinary()
+
+	keyTypeIsEmptyInterface := keyType.Kind() == Interface && keyType.NumMethod() == 0
+	shouldUnpackInterface := !keyTypeIsEmptyInterface && keyType.Kind() != String && !keyType.isBinary()
 
 	return &MapIter{
-		m:            v,
-		it:           hashmapNewIterator(),
-		keyInterface: isKeyStoredAsInterface,
+		m:                  v,
+		it:                 hashmapNewIterator(),
+		unpackKeyInterface: shouldUnpackInterface,
 	}
 }
 
@@ -1007,8 +1045,8 @@ type MapIter struct {
 	key Value
 	val Value
 
-	valid        bool
-	keyInterface bool
+	valid              bool
+	unpackKeyInterface bool
 }
 
 func (it *MapIter) Key() Value {
@@ -1016,7 +1054,7 @@ func (it *MapIter) Key() Value {
 		panic("reflect.MapIter.Key called on invalid iterator")
 	}
 
-	if it.keyInterface {
+	if it.unpackKeyInterface {
 		intf := *(*interface{})(it.key.value)
 		v := ValueOf(intf)
 		return v
@@ -1065,6 +1103,13 @@ func (v Value) Set(x Value) {
 	memcpy(v.value, xptr, size)
 }
 
+func (v Value) SetZero() {
+	v.checkAddressable()
+	v.checkRO()
+	size := v.typecode.Size()
+	memzero(v.value, size)
+}
+
 func (v Value) SetBool(x bool) {
 	v.checkAddressable()
 	v.checkRO()
@@ -1566,6 +1611,9 @@ func (e *ValueError) Error() string {
 //go:linkname memcpy runtime.memcpy
 func memcpy(dst, src unsafe.Pointer, size uintptr)
 
+//go:linkname memzero runtime.memzero
+func memzero(ptr unsafe.Pointer, size uintptr)
+
 //go:linkname alloc runtime.alloc
 func alloc(size uintptr, layout unsafe.Pointer) unsafe.Pointer
 
@@ -1638,7 +1686,7 @@ func buflen(v Value) (unsafe.Pointer, uintptr) {
 func sliceGrow(buf unsafe.Pointer, oldLen, oldCap, newCap, elemSize uintptr) (unsafe.Pointer, uintptr, uintptr)
 
 // extend slice to hold n new elements
-func (v *Value) extendSlice(n int) {
+func extendSlice(v Value, n int) sliceHeader {
 	if v.Kind() != Slice {
 		panic(&ValueError{Method: "extendSlice", Kind: v.Kind()})
 	}
@@ -1661,14 +1709,11 @@ func (v *Value) extendSlice(n int) {
 		ncap = old.cap
 	}
 
-	newslice := sliceHeader{
+	return sliceHeader{
 		data: nbuf,
 		len:  nlen + uintptr(n),
 		cap:  ncap,
 	}
-
-	v.flags = valueFlagExported
-	v.value = (unsafe.Pointer)(&newslice)
 }
 
 // Append appends the values x to a slice s and returns the resulting slice.
@@ -1678,7 +1723,9 @@ func Append(v Value, x ...Value) Value {
 		panic(&ValueError{Method: "Append", Kind: v.Kind()})
 	}
 	oldLen := v.Len()
-	v.extendSlice(len(x))
+	newslice := extendSlice(v, len(x))
+	v.flags = valueFlagExported
+	v.value = (unsafe.Pointer)(&newslice)
 	for i, xx := range x {
 		v.Index(oldLen + i).Set(xx)
 	}
@@ -1713,6 +1760,27 @@ func AppendSlice(s, t Value) Value {
 	}
 }
 
+// Grow increases the slice's capacity, if necessary, to guarantee space for
+// another n elements. After Grow(n), at least n elements can be appended
+// to the slice without another allocation.
+//
+// It panics if v's Kind is not a Slice or if n is negative or too large to
+// allocate the memory.
+func (v Value) Grow(n int) {
+	v.checkAddressable()
+	if n < 0 {
+		panic("reflect.Grow: negative length")
+	}
+	if v.Kind() != Slice {
+		panic(&ValueError{Method: "Grow", Kind: v.Kind()})
+	}
+	slice := (*sliceHeader)(v.value)
+	newslice := extendSlice(v, n)
+	// Only copy the new data and cap: the len remains unchanged.
+	slice.data = newslice.data
+	slice.cap = newslice.cap
+}
+
 //go:linkname hashmapStringSet runtime.hashmapStringSetUnsafePointer
 func hashmapStringSet(m unsafe.Pointer, key string, value unsafe.Pointer)
 
@@ -1846,6 +1914,17 @@ func (v Value) FieldByName(name string) Value {
 	return Value{}
 }
 
+func (v Value) FieldByNameFunc(match func(string) bool) Value {
+	if v.Kind() != Struct {
+		panic(&ValueError{"FieldByName", v.Kind()})
+	}
+
+	if field, ok := v.typecode.FieldByNameFunc(match); ok {
+		return v.FieldByIndex(field.Index)
+	}
+	return Value{}
+}
+
 //go:linkname hashmapMake runtime.hashmapMakeUnsafePointer
 func hashmapMake(keySize, valueSize uintptr, sizeHint uintptr, alg uint8) unsafe.Pointer
 
